#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>


struct dungeon_map {
    unsigned char *data;
    int x, y;
};

static int get_room(const struct dungeon_map *map, int x, int y)
{
    int byte_n, bit_n;
    if(x < 0 || x >= map->x || y < 0 || y > map->y)
        return 0;

    byte_n = y * (map->x/8) + x/8;
    bit_n = 7 - x % 8;
    return (map->data[byte_n] & (1<<bit_n)) ? 1 : 0;
}

static unsigned int isqrt(unsigned int i)
{
    int n;
    if(i < 2)
        return i;
    n = (isqrt(i >> 2) << 1) + 1;
    return n*n > i ? n - 1 : n;
}


static int read_data(unsigned char **p, int *n)
{
    enum { chunk_size = 128 };
    struct chunk {
        unsigned char data[chunk_size];
        int used;
        struct chunk *next;
    };

    struct chunk *first = NULL;
    struct chunk *tmp;
    int c, prev;
    int total = 0;
    int line = 1;
    int idx;

    while((c = getchar()) != EOF) {
        if(isspace(c)) {
            if(c == '\n')
                line++;
            continue;
        }
        if(c >= '0' && c <='9')
            c -= '0';
        else
        if(c >= 'a' && c <='f')
            c = c - 'a' + 10;
        else
        if(c >= 'A' && c <='F')
            c = c - 'A' + 10;
        else {
            fprintf(stderr, "line %d: illegal char <<%c>>\n", line, c);
            continue;
        }

        if((total & 1) == 0) {
            prev = c;
        } else {
            if(!first || first->used >= chunk_size) {
                tmp = malloc(sizeof(*tmp));
                tmp->used = 0;
                tmp->next = first;
                first = tmp;
            }
            /* now the room is guaranteed */
            tmp->data[tmp->used] = (prev << 4 | c);
            tmp->used++;
        }
        total++;
    }

    total = (total+1) / 2;  /* now it is in bytes */
    *n = total;
    *p = malloc(total);

    idx = total;
    while(first) {
        idx -= first->used;
        memcpy(*p + idx, first->data, first->used);
        tmp = first;
        first = first->next;
        free(tmp);
    }

    return 1;
}

static int read_map(struct dungeon_map *map)
{
    unsigned char *p;
    int n, dim;
    if(!read_data(&p, &n))
        return 0;

    dim = isqrt(n * 8);
    
    if(dim * dim != n * 8) {
        fprintf(stderr, "Illegal byte count %d can't build map\n", n);
        return 0;
    }

    map->data = p;
    map->x = dim;
    map->y = dim;

    return 1;
}

static int check_hall(const struct dungeon_map *map, int x, int y,
                      int *n, int *m)
{
    int i, j, x1, y1;
    if(x < 1 || y < 1 || x >= map->x-1 || y >= map->y-1)
        return 0;
    if(get_room(map, x, y))
        return 0;
    if(!get_room(map, x-1, y))
        return 0;
    if(!get_room(map, x, y-1))
        return 0;
    if(!get_room(map, x-1, y-1))
        return 0;

    x1 = x+1;
    while(x1 < map->x && !get_room(map, x1, y)) {
        if(!get_room(map, x1, y-1))
            return 0;
        x1++;
    }
    x1--;

    y1 = y+1;
    while(y1 < map->y && !get_room(map, x, y1)) {
        if(!get_room(map, x-1, y1))
            return 0;
        y1++;
    }
    y1--;

    if(x1 >= map->x-1 || y1 >= map->y-1)
        return 0;

    for(i = x+1; i <= x1; i++)
        for(j = y+1; j <= y1; j++)
            if(get_room(map, i, j))
                return 0;

    for(i = x-1; i <= x1+1; i++)
        if(!get_room(map, i, y1+1))
            return 0;
    for(j = y-1; j <= y1+1; j++)
        if(!get_room(map, x1+1, j))
            return 0;

    *n = x1 - x + 1;
    *m = y1 - y + 1;

    return 1;
}

static int find_largest_hall(const struct dungeon_map *map,
                             int *x, int *y, int *dx, int *dy)
{
    int largest = 0, larg_x, larg_y, larg_m, larg_n;
    int i, j, m, n;

    for(i = 1; i < map->x - 1; i++)
        for(j = 1; j < map->y - 1; j++)
            if(check_hall(map, i, j, &m, &n) && m*n > largest) {
                largest = m*n;
                larg_x = i;
                larg_y = j;
                larg_m = m;
                larg_n = n;
            }
    if(!largest)
        return 0;
    *x = larg_x;
    *y = larg_y;
    *dx = larg_m;
    *dy = larg_n;
    return 1;
}

static void print_map(const struct dungeon_map *map)
{
    int i, j;
    for(j = 0; j < map->y; j++) {
        putchar('@');
        putchar(' ');
        for(i = 0; i < map->x; i++)
            putchar(get_room(map, i, j) ? '#' : '.');
        putchar('\n');
    }
}

int main()
{
    struct dungeon_map map;
    int x, y, m, n;
    if(!read_map(&map))
        return 1;
    //printf("@ %dx%d\n", map.x, map.y);
    /*
    if(map.x <= 80)
        print_map(&map);
    */
    if(!find_largest_hall(&map, &x, &y, &m, &n)) {
        fprintf(stderr, "No hall found\n");
        return 1;
    }
    printf("%d %d %d %d\n", x, y, m, n);
    return 0;
}
